﻿<?php
/**
 * Plugin Name: wpco.in Service
 * Plugin URI: http://wpco.in
 * Description: wpco.in Service adiciona uma URL encurtada no formato wpco.in em todos os posts.
 * Version: 1.5
 * Author: Gustavo Marttos
 * Author URI: http://www.wpcore.com.br
 */

class wpco_Service {

	/* Construct PHP5. */
	function __construct() {

		/* Corta o circuito para criar o link encurtado. */
		add_filter( 'pre_get_shortlink', array( &$this, 'wpco_get_shortlink' ), 10, 4 );
		/* Cria um botão no menu de administração (WP 3.1+). */
		add_action( 'admin_bar_menu', array( &$this, 'wpco_admin_bar_menu' ), 99 );
		/* Cria as funções do administrador */
		add_action( 'admin_init', array( &$this, 'wpco_admin' ) );
		/* Desativação do plugin */
		register_deactivation_hook( __FILE__, array( &$this, 'wpco_desactivation' ) );
	}

	/**
	 * Esta função chama o shortlink obtido. */
	function wpco_get_shortlink( $shortlink, $id, $context, $allow_slugs ) {

		/* Omite o shortlink na página frontal. */
		if ( is_front_page() )
			return false;

		global $wp_query;

		$post_id = '';

		/* Pega a ID das páginas, posts, post types. */
		if ( 'query' == $context && is_singular() )
			$post_id = $wp_query->get_queried_object_id();

		else if ( 'post' == $context ) {
			$post = get_post( $id );
			$post_id = $post->ID;
		}

		/* Confere o shortlink e confirma se já existe ou não */
		if ( $shortlink = get_metadata( 'post', $post_id, '_wpco_shortlink', true ) )
			return $shortlink;

		/* Obtem a URL do post. */
		$url = get_permalink( $post_id );

		/* Obtem o shortlink. */
		$shortlink = do_shortlink( $url );

		/* Guarda a URL no banco de dados... como cache :P */
		if ( !empty( $shortlink ) ) {
			update_metadata( 'post', $post_id, '_wpco_shortlink', $shortlink );
			return $shortlink;
		}

		/* Se o shortlink não foi gerado, retorna um valor nulo */
		return false;
	}

	/* Coloca o menu do shortlink na barra de administração (WP 3.1+) */
	function wpco_admin_bar_menu() {
		global $wp_admin_bar;

		/* Obtem o shortlink. */
		$shortlink = wp_get_shortlink( 0, 'query' );

		/* Se o shortink não existem em determinada página, retorna um valor nulo. */
		if ( empty( $shortlink ) )
			return false;

		/* Cria o menu "shortlink" */
		$wp_admin_bar->remove_menu( 'get-shortlink' );
		$wp_admin_bar->add_menu( array( 'id' => 'shortlink', 'title' => __( 'Shortlink' ), 'href' => $shortlink ) );

		/* Adicione o compartilhamento para o Twitter */
		$twitter = sprintf( 'http://twitter.com/?status=%1$s', str_replace( '+', '%20', urlencode( get_the_title() . ' - ' . $shortlink ) ) );
		$wp_admin_bar->add_menu( array( 'parent' => 'shortlink', 'id' => 'wpco-share', 'title' => __( 'Partilhar no Twitter', 'wpco-service' ), 'href' => $twitter, 'meta' => array( 'target' => '_blank' ) ) );
	}

	/* Carrega as funções do administrador. */
	function wpco_admin() {

		if ( is_admin() ) {

			/* Elimina o cache quando um post é atualizado. */
			add_action( 'save_post', array( &$this, 'wpco_cache_delete' ) );
			add_action( 'added_post_meta', array( &$this, 'wpco_cache_delete' ) );
			add_action( 'updated_post_meta', array( &$this, 'wpco_cache_delete' ) );
			add_action( 'deleted_post_meta', array( &$this, 'wpco_cache_delete' ) );
		}
	}

	/* Elimina todo o cache. Esta função é chamada para desativar o plugin e elimina todos os registros criados no banco de dados. */
	function wpco_desactivation() {

		/* Elimina todos os registros criados. */
		delete_metadata( 'post', false, '_wpco_shortlink', '', true );
	}

	/* Elimina o cache de determinado post. */
	function wpco_cache_delete( $post_id ) {
		delete_metadata( 'post', $post_id, '_wpco_shortlink' );
	}
}

/* Cria um novo exemplo da classe WPCO SERVICE. */
$bitly = new wpco_Service();

/* Cria um pedido ao wpco.in para obter o shortlink usando a API do WordPress. */
function do_shortlink( $url ) {

	/* Variavel inicializada. */
	$shortlink = '';

	/* Da formato a URL. */
	$url = urlencode( $url );

	/* Realiza o pedido para encurtar em wpco.in */
	$wpco = "http://wpco.in/api.php?url={$url}";

	/* Obtem a resposta de wpco.in */
	$response = wp_remote_get( $wpco );

	if ( !is_wp_error( $response ) && 200 == $response['response']['code'] )
		$shortlink = $response['body'];

	return $shortlink;
}